#!/bin/sh
#
# Webif^2 utility functions - X-Wrt
#
# This file is compatible with White Russian and Kamikaze.
#

#
# Misc. functions
#

# workarounds for stupid busybox slowness on [ ]
empty() {
	case "$1" in
		"") return 0 ;;
		*) return 255 ;;
	esac
}

equal() {
	case "$1" in
		"$2") return 0 ;;
		*) return 255 ;;
	esac
}

neq() {
	case "$1" in
		"$2") return 255 ;;
		*) return 0 ;;
	esac
}

# very crazy, but also very fast :-)
exists() {
	( < $1 ) 2>&-
}

fix_symlink_hack() {
	! touch "$1" >&- 2>&- && {
		local atmpfile
		atmpfile=$(mktemp "/tmp/webif-XXXXXX")
		cp -p "$1" "$atmpfile"
		equal "$?" "0" && {
			rm "$1"
			mv "$atmpfile" "$1"
		}
	}
}

remove_lines_from_file() {
	# $1=filename
	# $2=substring match indicating lines to remove (case sensitive)
	cat "$1" 2>/dev/null | grep -q "$2" && {
		fix_symlink_hack "$1"
		local _substr_sed
		_substr_sed=$(echo "$2" |  sed s/'\/'/'\\\/'/g)
		cat "$1" |  sed /"$_substr_sed"/d > "$1"
	}
}

# mktemp replacement that doesn't actually create the file (as busybox 1.3.1+ does)
mkuniqfilename() {
	local _lfile
	_lfile=$(mktemp $*)
	rm -f "$_lfile" 2>&- >&-
}

#
# Original config functions
#
#  These work with as a tuple based configuration system. apply.sh applies
#  the changes.
#
load_settings() {
	. /lib/webif/config.sh
	if [ $# -eq 1 ]; then
		exists $SBC_CONFIGDIR/$1.conf && . $SBC_CONFIGDIR/$1.conf
		exists /tmp/.config/$1.conf && . /tmp/.config/$1.conf
	fi
}
save_setting() {
	exists /tmp/.config/* || mkdir -p /tmp/.config
	oldval="$(eval "echo \${$2}")"
	grep "^$2=" /tmp/.config/$1.conf >&- 2>&- && {
		grep -v "^$2=" /tmp/.config/$1.conf > /tmp/.config/$1.conf.new 2>&- 
		mv /tmp/.config/$1.conf.new /tmp/.config/$1.conf 2>&- >&-
		oldval=""
	}
	equal "$oldval" "$3" || echo "$2=\"$3\"" >> /tmp/.config/$1.conf
}
#
# validate form variables using validate.awk
#
validate() {
	if empty "$1"; then
		eval "$(awk -f /usr/lib/webif/validate.awk)"
	else
		eval "$(echo "$1" | awk -f /usr/lib/webif/validate.awk)"
	fi
}

set_mtd() 
{
	mtd=$( cat /proc/mtd | grep "\"$1\"" | awk {'print $1'} )
	mtd=${mtd%:}
	mtd="/dev/$mtd"
}

set_mtdblock() 
{
	mtd=$( cat /proc/mtd | grep "\"$1\"" | awk {'print $1'} )
	mtd=${mtd%:}
	mtd=${mtd#mtd}
	mtd="/dev/mtdblock$mtd"
}

############################
# user app functions
############################

# $1 == app name
ua_make_config_file() {
	cat > $SBC_CONFIGDIR/ua_$1.conf << EOF
app_name="$1"
app_priority="20"
app_enabled="false"
app_daemonize="true"
app_cmd=""
app_args=""
EOF
}

# $1 == app name
ua_make_startup_file() {
	. $SBC_CONFIGDIR/ua_$1.conf
	cat > $SBC_CONFIGDIR/init.d/S$app_priority$1 << EOF
#!/bin/sh
#
# Start Custom app "$1"
#

# load config
. /lib/webif/config.sh
. \$SBC_CONFIGDIR/ua_$1.conf

app_file="\$SBC_USERAPPS/\$app_name/\$app_cmd"

if [ "\${app_cmd##*\.}" = "class" ]; then
	run_app="jamvm \${app_cmd%\\.class}"
elif [ "\${app_cmd##*\.}" = "jar" ]; then
	run_app="jamvm -jar \$app_cmd"
else
	run_app="\$app_file"
fi

run_app="\$run_app \$app_args"

if [ "\$app_daemonize" = "true" ]; then
	daemon="&"
fi

start()
{
	echo -n "Starting \$app_name: "

	mkdir -p /tmp/\$app_name;

	if [ "\$app_daemonize" = "true" ]; then
	    (
		cd \$SBC_USERAPPS/\$app_name; 
		exec > /tmp/\$app_name/stdout; 
		exec 2> /tmp/\$app_name/stderr;
		exec <&-
		\$run_app &
	    ) &
		wait
	else
	    (
		exec > /tmp/\$app_name/stdout; 
		exec 2> /tmp/\$app_name/stderr; 
		exec <&-
		cd \$SBC_USERAPPS/\$app_name; 
		\$run_app
	    )
	fi
}

stop()
{
	ps | grep -v grep | grep "\$run_app" | awk '{print \$1}' | xargs kill -KILL
}

case "\$1" in
  forcestart) echo -n "Starting \$app_name:: "
	start
	echo "OK"
	;;
  start) if [ "\$app_enabled" = "true" ]; then
		echo -n "Starting \$app_name:: "
		start
		echo "OK"
	fi
	;;
  stop) echo -n "Stopping \$app_name:: "
	stop
	echo "OK"
	;;
  restart) echo -n "Restarting \$app_name:: "
	stop
	sleep 1
	start
	echo "OK"
	;;
  *) echo "Usage: \$SCRIPTNAME {start|stop|restart}" >&2
	exit 1
	;;
esac

exit 0
EOF
chmod 755 $SBC_CONFIGDIR/init.d/S$app_priority$1
}
